--
-- nurseTanker
-- Class for nurse tanker
--
-- @author  PeterJ_FS-UK modteam
-- @date  28/11/2012
--
-- Copyright (C) GIANTS Software GmbH, Confidential, All Rights Reserved.
--
nurseTanker = {}

function nurseTanker.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Fillable, specializations) and SpecializationUtil.hasSpecialization(ImplementLinks, specializations);
end;

function nurseTanker:load(xmlFile)
    Sprayer.load(self, xmlFile);
    --
    self.setVehicleRpmUp = SpecializationUtil.callSpecializationsFunction("setVehicleRpmUp");
    self.saveMinRpm = 0; 
    self.printWarningTime = 0;
    self.printWarningTime1 = 0;
	
	--Trigger
	self.FillableTrigger = nurseTanker.FillableTrigger;
	self.FillableTriggers = {};
  
	local FillableTrigger = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.fillableTrigger#index"));
	if FillableTrigger ~= nil then
		self.FillableTriggers[FillableTrigger] = {node=FillableTrigger, pipeState=2};
	end;
	for _, FillableTrigger in pairs(self.FillableTriggers) do
		addTrigger(FillableTrigger.node, "FillableTrigger", self);
	end;
  
	self.isSpreaderInRange = nil;
	
	--Show fill level of machine in range
	self.hudFillLevelBasePosX = 0.01;
	self.hudFillLevelBasePosY = 0.0149;
	self.hudFillLevelBaseWidth = 0.2;
	self.hudFillLevelBaseHeight = 0.043;
	self.hudFillLevelBarWidth = 0.168;
	self.hudFillLevelOverlay = Overlay:new("hudFillLevelOverlay", "dataS2/menu/hud/vehicleHUD_barGold.png", self.hudFillLevelBasePosX + 0.0228, self.hudFillLevelBasePosY + 0.003, self.hudFillLevelBaseWidth - 0.03, self.hudFillLevelBaseHeight - 0.012);
	self.FillLevelBasePath = Utils.getFilename("textures/FillLevelBase.png", self.baseDirectory);
	self.hudFillLevelBase = Overlay:new("hudFillLevelBase", self.FillLevelBasePath, self.hudFillLevelBasePosX, self.hudFillLevelBasePosY, self.hudFillLevelBaseWidth, self.hudFillLevelBaseHeight);
	self.FillLevelFramePath = Utils.getFilename("textures/FillLevelFrame.png", self.baseDirectory);
	self.hudFillLevelFrame = Overlay:new("hudFillLevelFrame", self.FillLevelFramePath, self.hudFillLevelBasePosX, self.hudFillLevelBasePosY, self.hudFillLevelBaseWidth, self.hudFillLevelBaseHeight);

 	self.setVehicleRpmUp = SpecializationUtil.callSpecializationsFunction("setVehicleRpmUp");
	self.saveMinRpm = 0; 
    self.printWarningTime = 0;
    self.printWarningTime1 = 0;
	self.beaconLight = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.beaconLights.beaconLight#index1"));
	
	--- fill sound ---
    if self.isClient then
        local fillingSound = getXMLString(xmlFile, "vehicle.fillingSound#file");
        if fillingSound ~= nil and fillingSound ~= "" then
            fillingSound = Utils.getFilename(fillingSound, self.baseDirectory);
            self.fillingSound = createSample("fillingSound");
            self.fillingSoundEnabled = false;
            loadSample(self.fillingSound, fillingSound, false);
            self.fillingSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fillingSound#pitchOffset"), 1);
            self.fillingSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fillingSound#volume"), 1);
        end;
    end; 
end;

function nurseTanker:delete()
    Sprayer.delete(self);
	
    if self.fillingSound ~= nil then
        delete(self.fillingSound);
    end;
	for _, FillableTrigger in pairs(self.FillableTriggers) do
		removeTrigger(FillableTrigger.node);
	end;
end;

function nurseTanker:readStream(streamId, connection)
    Sprayer.readStream(self, streamId, connection);
end;

function nurseTanker:writeStream(streamId, connection)
    Sprayer.writeStream(self, streamId, connection);
end;

function nurseTanker:readUpdateStream(streamId, timestamp, connection)
    Sprayer.readUpdateStream(self, streamId, timestamp, connection);
end;

function nurseTanker:writeUpdateStream(streamId, connection, dirtyMask)
    Sprayer.writeUpdateStream(self, streamId, connection, dirtyMask)
end;

function nurseTanker:mouseEvent(posX, posY, isDown, isUp, button)
    Sprayer.mouseEvent(self, posX, posY, isDown, isUp, button);
end;

function nurseTanker:keyEvent(unicode, sym, modifier, isDown)
    Sprayer.keyEvent(self, unicode, sym, modifier, isDown);
end;
  
function nurseTanker:update(dt)

    if self.isClient then
        if self:getIsActiveForInput() then
            if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA) and not self.PTOId and self.attacherVehicle.isMotorStarted then
                  self:setIsTurnedOn(not self.isTurnedOn);
            end;
			if InputBinding.isPressed(InputBinding.IMPLEMENT_EXTRA) and self.PTOId then
				self.printWarningTime = self.time + 1000;
			end;	
			if InputBinding.isPressed(InputBinding.ACTIVATE_OBJECT) and self.PTOId then
				self.printWarningTime1 = self.time + 1000;
			end;
        end;
    end;
end;

function nurseTanker:updateTick(dt)
    if self.isClient then
        if self.PTOId or not self.attacherVehicle.isMotorStarted then
            -- PTO is detached, or engine is off.
            self:setIsTurnedOn(false);
            self:setIsSprayerFilling(false);
        end;
        self:setVehicleRpmUp(dt, self.isTurnedOn);
    end;
	
	--- fill sprayer in range ---
	--if self.isServer and self:getIsActive() then
		if self.isTurnedOn and self.isSpreaderInRange ~= nil then
			local fillable = self.isSpreaderInRange;
			if fillable.setFillLevel ~= nil then
				for fillType,v in pairs(fillable.fillTypes) do
					if fillType == self.currentFillType then
						if  fillable.fillLevel ~= fillable.capacity  then
							local litersPerSecond = self.sprayLitersPerSecond[self.currentFillType];
							if litersPerSecond == nil then
								litersPerSecond = self.defaultSprayLitersPerSecond;
							end;
							local usage = litersPerSecond * dt*0.001;
							fillable:resetFillLevelIfNeeded(Fillable.FILLTYPE_LIQUIDMANURE);
							fillable:setFillLevel(fillable.fillLevel + usage, Fillable.FILLTYPE_LIQUIDMANURE);
						end; 	
					end;
				end;
			end;
		end;
	--end;
    if self.isClient then		
		if self.isSprayerFilling then
			if not self.fillingSoundEnabled and self:getIsActiveForSound() then
                playSample(self.fillingSound, 0, self.fillingSoundVolume, 0);
                setSamplePitch(self.fillingSound, self.fillingSoundPitchOffset);
                self.fillingSoundEnabled = true;
            end;
        end;
		if not self.isSprayerFilling then
			stopSample(self.fillingSound);
			self.fillingSoundEnabled = false;
		end;			
	end;
	
    --
    Sprayer.updateTick(self, dt);
end;

function nurseTanker:draw()
    if self.isClient then
		if self.printWarningTime > self.time then
			g_currentMission:addWarning(g_i18n:getText("turnON_Error"), 0.018, 0.033);
		end;
		if self.printWarningTime1 > self.time then
			g_currentMission:addWarning(g_i18n:getText("fill_Error"), 0.018, 0.033);
		end;
    end;
	
	if self:getIsActive() and not g_currentMission.missionPDA.showPDA then
		if self.isSpreaderInRange ~= nil then
			local fillable = self.isSpreaderInRange;
			if self.isSpreaderInRange.fillLevel ~= nil then
				for fillType,v in pairs(fillable.fillTypes) do
					if fillType == self.currentFillType and fillable.rootNode ~= self.rootNode then
						self.hudFillLevelBase:render();
						local percent = (self.isSpreaderInRange.fillLevel / self.isSpreaderInRange.capacity) * 100;
						setTextBold(true);
						setTextAlignment(RenderText.ALIGN_CENTER);
						setTextColor(0, 0, 0, 1);
						renderText(self.hudFillLevelBasePosX + self.hudFillLevelBaseWidth / 2 + 0.003, self.hudFillLevelBasePosY + 0.0083, 0.024, string.format("%d (%d%%)", self.isSpreaderInRange.fillLevel, percent));
						setTextColor(1, 1, 1, 1);
						renderText(self.hudFillLevelBasePosX + self.hudFillLevelBaseWidth / 2 + 0.003, self.hudFillLevelBasePosY + 0.0113, 0.024, string.format("%d (%d%%)", self.isSpreaderInRange.fillLevel, percent));
						setTextBold(false);
						self.hudFillLevelOverlay.width = self.hudFillLevelBarWidth * (self.isSpreaderInRange.fillLevel / self.isSpreaderInRange.capacity);
						setOverlayUVs(self.hudFillLevelOverlay.overlayId, 0, 0.05, 0, 1, self.isSpreaderInRange.fillLevel / self.isSpreaderInRange.capacity, 0.05, self.isSpreaderInRange.fillLevel / self.isSpreaderInRange.capacity, 1);
						self.hudFillLevelOverlay:render();
						self.hudFillLevelFrame:render();
					end;
				end;
			end;
		end;
	end;
    --
	Sprayer.draw(self);
end;

function nurseTanker:onDetach()

    Sprayer.onDetach(self);
    --
    for k, steerable in pairs(g_currentMission.steerables) do
        if self.attacherVehicleCopy == steerable then
            steerable.motor.minRpm = self.saveMinRpm;
            self.attacherVehicleCopy = nil;
        end;
    end;
end;

function nurseTanker:onAttach(attacherVehicle)
    Attachable.onAttach(self, attacherVehicle);
    --
    if self.attacherVehicleCopy == nil then
        self.attacherVehicleCopy = self.attacherVehicle;
    end;
    self.saveMinRpm = self.attacherVehicle.motor.minRpm;
end;

function nurseTanker:onLeave()
    Sprayer.onLeave(self);
end;

function nurseTanker:onDeactivate()
    Sprayer.onDeactivate(self);
end;

function nurseTanker:onDeactivateSounds()
	if self.fillingSoundEnabled then
		stopSample(self.fillingSound);
		self.fillingSoundEnabled = false;
	end;
    Sprayer.onDeactivateSounds(self);
end;

function nurseTanker:getIsTurnedOnAllowed(isTurnedOn)
    return Sprayer.getIsTurnedOnAllowed(self, isTurnedOn);
end;

function nurseTanker:setIsTurnedOn(isTurnedOn, noEventSend)
    Sprayer.setIsTurnedOn(self, isTurnedOn, noEventSend);
end;

function nurseTanker:setIsSprayerFilling(isFilling, fillType, isSiloTrigger, noEventSend)
    Sprayer.setIsSprayerFilling(self, isFilling, fillType, isSiloTrigger, noEventSend);
end;

function nurseTanker:addSprayerFillTrigger(trigger)
    Sprayer.addSprayerFillTrigger(self, trigger);
end;

function nurseTanker:removeSprayerFillTrigger(trigger)
    Sprayer.removeSprayerFillTrigger(self, trigger);
end;

function nurseTanker:FillableTrigger(triggerId, otherId, onEnter, onLeave, onStay, otherShapeId)
	if onEnter or onLeave then
		local spreader = g_currentMission.nodeToVehicle[otherShapeId];
		if spreader ~= nil then
			if spreader ~= self and spreader.setFillLevel	then
				if onEnter then
					self.isSpreaderInRange = (spreader);
				else
					self.isSpreaderInRange = nil;
				end;
			end;
		end;
	end;
end;

function nurseTanker:setVehicleRpmUp(dt, isActive)
    if self.attacherVehicle ~= nil and self.saveMinRpm ~= 0 then
        if dt ~= nil then
            if isActive == true then
                self.attacherVehicle.motor.minRpm = math.max(self.attacherVehicle.motor.minRpm-dt, -1000);
            else
                self.attacherVehicle.motor.minRpm = math.min(self.attacherVehicle.motor.minRpm+dt*2, self.saveMinRpm);
            end;
        else
            self.attacherVehicle.motor.minRpm = self.saveMinRpm;
        end;
        if self.attacherVehicle.isMotorStarted then
            local fuelUsed = 0.0000011*math.abs(self.attacherVehicle.motor.minRpm);
            self.attacherVehicle:setFuelFillLevel(self.attacherVehicle.fuelFillLevel-fuelUsed);
            g_currentMission.missionStats.fuelUsageTotal = g_currentMission.missionStats.fuelUsageTotal + fuelUsed;
            g_currentMission.missionStats.fuelUsageSession = g_currentMission.missionStats.fuelUsageSession + fuelUsed;
        end;
    end;
end;
